/*

  pot est un programme servant  lister vos "pot" sur un rseau
  Copyright (C) 2001 Fabien Carrion et Guillaume Pinot
  
  Ce programme est libre, vous pouvez le redistribuer et/ou le
  modifier selon les termes de la Licence Publique Gnrale GNU
  publie par la Free Software Foundation (version 2).

  Ce programme est distribu car potentiellement utile, mais SANS
  AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties
  de commercialisation ou d'adaptation dans un but
  spcifique. Reportez-vous  la Licence Publique Gnrale
  GNU pour plus de dtails.

  Vous devez avoir reu une copie de la Licence Publique
  Gnrale GNU en mme temps que ce programme ; si ce n'est
  pas le cas, crivez  la Free Software Foundation, Inc., 59
  Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.

  fcarrion@etu.utc.fr
  pinotgui@etu.utc.fr

*/

#include "commun.h"
#include "kelsal.h"
#include "readutmp.h"

/* affiche un fichier sur stdout */
void view(char *chemin)
{
  /* initialisation des variables */
  int n = 0;
  char nom[10];
  FILE *pot;

  pot = fopen(chemin, "r");
  erreur(pot);

  /* affichage */
  while(fscanf(pot,"%9s",nom) != EOF)
    {
      n++;
      printf("%-13s",nom);
      if (n > 5)
        {
		n = 0;
		printf("\n");
        }
    }
  fclose(pot);
  printf("\n");
}


/* supprime un login de ~/.pot.pot */
void suppr(char *chemin,void *del)
{
  struct liste_pot *premier = charge_liste_pot(chemin);

  premier = del_liste_pot(del, premier);
  fichier_liste_pot(chemin,premier);
  suppr_all_liste_pot(premier);

  return ;
}


/* gestion des erreurs  l'ouverture d'un flux */
void erreur(FILE *tube)
{
  if (!tube)
    {
      fprintf( stderr , "Erreur : %s\n" , strerror(errno));
      exit(0);
    }
}


struct liste_utmp * ajout_liste_p_presentes(STRUCT_UTMP *description,
					     struct liste_utmp *p_presentes)
{
  struct liste_utmp *previous = NULL;
  struct liste_utmp *current = p_presentes;

  struct liste_utmp *new;

  if( !(new=malloc(sizeof(struct liste_utmp))))
    {
	 fprintf(stderr,
		 "ajout_liste_p_presentes : "
		 "Allocation de memoire impossible\n");
	 return NULL;
    }

  if (!(new->description = (STRUCT_UTMP *)malloc(sizeof(STRUCT_UTMP))))
    {
	 fprintf(stderr,
		 "ajout_liste_p_presentes : "
		 "Allocation de memoire impossible\n");
	 return NULL;
    }

  *(new->description) = *description;

  new->next = NULL;

  while(current != NULL)
    {
	 if(strcmp(description->ut_user, current->description->ut_user) > 0)
	   {
		previous = current;
		current = current->next;
	   }
	 else
	   break;
    }

  if(previous == NULL) /* on ajoute le login en tete */
    {
	 new->next = current;
	 p_presentes = new;
    }
  else
    {
	 previous->next = new;
	 new->next = current;
    }

  return p_presentes;
}

int add_pots_utmp(STRUCT_UTMP *desc)
{
  return ajouteliste(desc->ut_user, 
		     desc->ut_host, 
		     desc->ut_line, 
		     getpwnam(desc->ut_user));
}

int add_pots_samba(struct liste_samba *desc)
{
  return ajouteliste(desc->login, 
		     desc->ordi, 
		     "smb", 
		     getpwnam(desc->login));
}


void affiche_samba(struct liste_samba *list)
{
  while(list != NULL)
    {
      printf("login_samba = %s\n", list->login);
      list = list->next;
    }
}

void affiche_utmp(struct liste_utmp *list)
{
  while(list != NULL)
    {
      printf("login_utmp = %s\n", list->description->ut_user);
      list = list->next;
    }
}

/*

  Les deux listes sont classees par ordres alphabetique donc la
  recherche des pots presents peut se faire en une seule passe

  Pour cela, on compare les deux noms d'utilisateur dans les deux
  listes.

  - Si le nom dans p_presentes est le meme que celui dans pots, on
    ajoute la personne dans la liste des presents.  Et on avance dans
    p_presentes (au cas ou la personne est loguee plusieurs fois).

  - Si le nom dans p_presentes est "inferieur" (placer avant dans
    l'ordre alphabetique), on avance dans la liste p_presentes

  - A l'inverse, si le nom dans p_presentes est "superieur" on avance
    dans pots.

*/


#ifdef SAMBA
int cherche_pots (struct liste_utmp *p_presentes, 
		  struct liste_samba *p_samba,
		  struct liste_pot *pots)
#else
int cherche_pots (struct liste_utmp *p_presentes, struct liste_pot *pots)
#endif
{
  int ReturnValUtmp = 1;
  int ReturnValSamba = 1;
  int compteur = 0;

  //  affiche_utmp(p_presentes);
  //  affiche_samba(p_samba);

  /* pas la peine de continuer si l'une des listes est vide */
#ifndef SAMBA
  while (p_presentes != NULL && pots != NULL)
#else
    while ((p_presentes != NULL || p_samba != NULL ) && pots != NULL)
#endif
      {
	if(p_presentes != NULL)
	  {
	    ReturnValUtmp = strcmp(p_presentes->description->ut_user,
				   pots->login);
	    //	    printf("%s(%d) ",p_presentes->description->ut_user,ReturnValUtmp); 
	  }
	else
	  ReturnValUtmp = 1;
	
#ifdef SAMBA
	if(p_samba != NULL)
	  {
	    ReturnValSamba = strncmp(p_samba->login, pots->login,9);
	    //printf("%s(%d) ", p_samba->login, ReturnValSamba);
	  }
	else
	  ReturnValSamba = 1;


#endif

	
	if(ReturnValSamba > 0 && ReturnValUtmp > 0)
	  {
	    pots = pots->next;
	    //	  continue;
	  }

	//	printf("%s", pots->login);

	
#ifdef SAMBA
	if (ReturnValSamba == 0)
	  {
	    add_pots_samba(p_samba);
	    //printf(" ajout samba");
	    compteur++;
	  }
#endif
	
	if(ReturnValUtmp == 0)
	  {
	    add_pots_utmp(p_presentes->description);
	    //printf(" ajout utmp");
	    compteur++;
	  } 

	//	printf("\n");

#ifdef SAMBA      
	if(ReturnValSamba <= 0)
	  p_samba = p_samba->next;
#endif
	
	if(ReturnValUtmp <= 0)
	  p_presentes = p_presentes->next;
	
	
      }

  //  printf("Fin cherche_pot\n");
  
  return compteur;
} 



/* remplit la stucture liste_connecte par rapport  localhost,
renvoie le nombre de "pot", -1 si echoue */

struct liste_utmp * finger()
{
  /*initialisation des variables */
  int n_users;
  STRUCT_UTMP *utmp_buf;
  
  //  struct liste_pot *pots;
  struct liste_utmp *p_presentes = NULL;
  
  
  /* Reutilisation de readutmp de GNU sh-utils */

  /* ouverture des flux */
  if (read_utmp (UTMP_FILE, &n_users, &utmp_buf))
    {
      fprintf(stderr,"Erreur  l'ouverture de UTMP\n");
      return (struct liste_utmp *)NULL;
    }


  /* lecture d'utmp */
  while (n_users--)
    {
      if (utmp_buf[n_users].ut_name[0]
#ifdef USER_PROCESS
	  && utmp_buf[n_users].ut_type == USER_PROCESS
#endif
	  )
	{	  
	  /* La personne est logue sur le system. On l'ajoute a la
	   liste des personnes presentes de facon trie (ordre
	   alphabetique) */

	  p_presentes = ajout_liste_p_presentes(&(utmp_buf[n_users]), p_presentes);

	}
    }

  free(utmp_buf);

  return p_presentes;
}


#ifdef SAMBA


/* fonction ajoutant un login dans liste_samba dans une liste chaine
 simple trie par ordre alphabetique */
struct liste_samba *ajout_liste_samba(char *login,
				      char *ordi,
				      struct liste_samba *premier)
{
  struct liste_samba *previous = NULL;
  struct liste_samba *current = premier;

  struct liste_samba *new;

  if(!(new = malloc(sizeof(struct liste_samba))))
    {
      printf("Erreur d'allocation ajout_liste_samba\n");
      return NULL;
    }

  if(!(new->login = malloc(sizeof(char)*10)))
    {
      printf("Erreur d'allocation ajout_liste_samba\n");
      return NULL;
    }

  if(!(new->ordi = malloc(sizeof(char)*10)))
    {
      printf("Erreur d'allocation ajout_liste_samba\n");
      return NULL;
    }

  strncpy(new->login,login,9);
  strncpy(new->ordi,ordi,10);

  new->next = NULL;

  /* Recherche de l'emplacement du nouveau login */
  /* On avance jusqu'a trouve soit la fin, soit un login qui est
     superieur au nouveau */

  while(current != NULL)
    {
	 if(strcmp(login, current->login) > 0)
	   {
		previous = current;
		current = current->next;
	   }
	 else
	   break;
    }

  if(previous == NULL) /* on ajoute le login en tete */
    {
	 new->next = current;
	 premier = new;
    }
  else
    {
	 previous->next = new;
	 new->next = current;
    }

  return premier;
}

struct liste_samba *samba()
{
  char login[15];
  char ordi[15];
  struct liste_samba *lsamba = NULL;

  FILE *smb;

  smb = (FILE *) popen("smbstatus -b | cut -c9-28 | tail +5","r");

  if(!smb)  
    return (struct liste_samba *)-1;


  /* lecture du flux */


  while( (fscanf(smb,"%s", login) != EOF) && (fscanf(smb,"%s", ordi) != EOF))
    {
      if ( ! (lsamba = ajout_liste_samba (login, ordi, lsamba)))
	return (struct liste_samba *)-1;      
    }
  pclose(smb);

  return lsamba;

}

#endif

void freelistutmp(struct liste_utmp *list)
{
  struct liste_utmp *next;

  while(list != NULL)
    {
      next = list->next;
      free(list->description);
      free(list);
      list = next;
    }
}

void freelistsamba(struct liste_samba *list)
{
  struct liste_samba *next;

  while(list != NULL)
    {
      next = list->next;
      free(list->login);
      free(list->ordi);
      free(list);
      list = next;
    }
}

void freelistpots(struct liste_pot *list)
{
  struct liste_pot *next;

  while(list != NULL)
    {
      next = list->next;
      free(list);
      list = next;
    }
}

int recherche_pots(char *chemin)
{
  int compteur = 0;

  struct liste_pot *lpot;
  struct liste_utmp *lutmp = (struct liste_utmp *)-1;

#ifdef SAMBA
  struct liste_samba *lsamba = (struct liste_samba *)-1;
#endif


  lpot = charge_liste_pot(chemin);

  if(!lpot)
    printf("Echec du chargement de la liste de pot\n");

  lutmp = finger();

  if (lutmp == (struct liste_utmp *)-1)
    printf("Erreur lors de la recherche de personnes connectees\n");

#ifdef SAMBA

  lsamba = samba();

  if (lsamba == (struct liste_samba *)-1)
    printf("Erreur lors de la recherche de personnes connectees (PC)\n");

#endif


  compteur = cherche_pots (lutmp, 
#ifdef SAMBA
		       lsamba,
#endif
			  lpot);


  freelistutmp(lutmp);

#ifdef SAMBA
  freelistsamba(lsamba);
#endif

  freelistpots(lpot);

  return compteur;
}


/* la fonction rajoute un connecte, 0 quand choue, 1 sinon */
int ajouteliste(char *login , char *ordi , char *tty , struct passwd *ki)
{
  struct liste_connecte *connecte;
  struct liste_connecte *cur;

  /* Creation de la structure suivante */

  if ( ( connecte = (struct liste_connecte *) malloc(sizeof(struct liste_connecte)) ) == NULL )
    {
      fprintf ( stderr , "Allocation de memoire impossible\n" );
      return 0;
    }

  /* Ajout en fin de liste pour preserver l'ordre alphabetique */

  cur = &depart;

  while (cur->next != NULL)
    cur = cur->next;

  cur->next = connecte;
  connecte->next = NULL;

  //  connecte->next = depart.next;
  //depart.next = connecte;
  //connecte->previous = &depart;
  //if( connecte->next != NULL)
  //  connecte->next->previous = connecte;

  /* remplissage de la structure */

  /* login */
  if ( (connecte->login = (char *) calloc(strlen(login) + 1, sizeof(char)))
       == NULL )
    {
      fprintf ( stderr , "Allocation de memoire impossible\n" );
      return 0;
    }
  strcpy(connecte->login,login);

  /* ordi */
  if ( (connecte->ordi = (char *) calloc(strlen(ordi) + 1, sizeof(char)))
       == NULL )
    {
      fprintf ( stderr , "Allocation de memoire impossible\n" );
      return 0;
    }
  strcpy(connecte->ordi,ordi);
  
  /* nom */
  if ( ki->pw_gecos[0] != ',' && ki->pw_gecos[0] != '\0' )
    {
      if ( (connecte->nom = (char *) calloc(strlen(ki->pw_gecos) + 1,
					    sizeof(char)))
	   == NULL )
        {
	  fprintf ( stderr , "Allocation de memoire impossible\n" );
	  return 0;
        }
      strcpy( connecte->nom,strtok(ki->pw_gecos,",") );
    }
  else
    {
      if ( ( connecte->nom = (char *) malloc (sizeof("???")) ) == NULL )
        {
	  fprintf ( stderr , "Allocation de memoire impossible\n" );
	  return 0;
        }
      strcpy(connecte->nom,"???");
    }

  
  /* talk */
  connecte->talk = talkable(tty);
  
  /* idle */
  connecte->idle = idle_time(tty);

  /* salle */
  if ( (connecte->salle = kelsal(ordi)) == NULL )
    {
      fprintf ( stderr , "Probleme avec le fichier de configuration\n" );
      return 0;
    }
  
  return 1;
}

/* la fonction retourne 1 si on peut communiquer avec la personne sinon 0 */
short talkable( char *tty )
{
  /* initialisation des variables */
  struct stat ttystatus;
  char *buffer = NULL;

  /* Si sur samba */
  if ( ! strcmp(tty,"smb") )
    return 2;

  /* Si xdm */
  if (tty[0] == ':')
    return 3;

  if (tty[0]!='/')
    {
      /* recuperation du nom de la console */
      if ( ( buffer = (char *)malloc((strlen(tty)+6)*sizeof(char)) ) == NULL )
	{      
	  fprintf ( stderr , "Allocation de memoire impossible\n" );
	  return 0;
	}

      strcpy( buffer , "/dev/" );
      strcat(buffer,tty);
    }
  else
    if ( (buffer=strdup(tty)) == NULL )
      {      
	fprintf ( stderr , "Allocation de memoire impossible\n" );
	return 0;
      }
  

  /* recuperation du mode d'ecriture de la console */
  if (stat(buffer, &ttystatus) < 0)
    {
      if(buffer != NULL)
	free(buffer);
      return 0;
    }
  

  if(buffer != NULL)
    free(buffer);

  /* retour de la fonction : console ecrivable ou non */
  if( (ttystatus.st_mode & TALKABLE) == TALKABLE )
    return 1;
  else
    return 0;
}

/* retourne l'idle time, -1 si erreur, -2 si samba, -3 si xdm */
time_t idle_time(char *tty)
{
  struct stat stats;
  time_t last_change = 0;
  time_t now = 0;
  char *buffer = NULL;

  if ( ! strcmp(tty,"smb") )
    return -2;

  if (tty[0] == ':')
    return -3;

  if (tty[0]!='/')
    {
      /* recuperation du nom de la console */
      if ( ( buffer = (char *)malloc((strlen(tty)+6)*sizeof(char)) ) == NULL )
	{      
	  fprintf ( stderr , "Allocation de memoire impossible\n" );
	  return -1;
	}

      strcpy( buffer , "/dev/" );
      strcat(buffer,tty);
    }
  else
    if ( (buffer=strdup(tty)) == NULL )
      {      
	fprintf ( stderr , "Allocation de memoire impossible\n" );
	return -1;
      }
  

  /* recuperation du mode d'ecriture de la console */
  if (stat (buffer, &stats) == 0)
    {
      last_change = stats.st_atime;
      time (&now);
    }
  else
    return -1;

  if(buffer != NULL)
    free(buffer);

  /* retour de la fonction : console ecrivable ou non */
  return now - last_change;
}



/* efface la liste chaine */
int free_liste( void )
{
  struct liste_connecte *cur,*next;
 
  //  return 0;
 
  if ( (cur = depart.next) == NULL )
    return 0;
  
  while (cur != NULL)
    {
      free(cur->login);
      free(cur->ordi);
      free(cur->nom);
      free(cur->salle);

      next = cur->next;

      free(cur);

      cur=next;
    }

  depart.next = NULL;

  return 0;
}

/* Verification du login entree : renvoie 0 s il est correct ; -1 ou
   -2 sinon */
short int verif_login(char *login,
		      int choix,
		      char *chemin,
		      struct liste_pot *premier)
{
  char nom[10];
  FILE *pot;
  struct liste_pot *cherche;

  if ( choix == 1 )
    {
      pot = fopen(chemin, "r");
      erreur(pot);
      
      while(fscanf(pot,"%9s",nom) != EOF)
	   if( strcmp(nom , login) == (int)NULL) 
		return -2;
  
      fclose(pot);
    }
  else if ( choix == 2 )
    {
      cherche = premier;
  
      if ( ! strcmp(cherche->login,login) )
	return -2;
      else
	{
	  while ( (cherche->next != NULL)
		  && strcmp(cherche->next->login,login) )
	    cherche=cherche->next;
      
	  if (cherche->next != NULL)
	    return -2;
	}
    }

  if (getpwnam(login) == NULL)
    return -1;
  else
    return 0;
}

/* fonction chargeant les personnes du .pot.pot dans la liste chainee
   liste_pot */
struct liste_pot *charge_liste_pot(char *chemin)
{
  FILE *pot;
  struct liste_pot *premier;
  char login[9];
  

  pot = fopen(chemin, "r");
  erreur(pot);

  premier=malloc(sizeof(struct liste_pot));
  fscanf(pot,"%9s",login);
  strncpy(premier->login,login,9);
  premier->next = NULL;
  

  while(fscanf(pot,"%9s",login) != EOF)
    {
      premier = ajout_liste_pot(login,premier);
    }

  fclose(pot);

  return premier;
}

/* fonction ajoutant un login dans liste_pot dans une liste chaine
   simple trie par ordre alphabetique */
struct liste_pot *ajout_liste_pot(char *login, struct liste_pot *premier)
{
  struct liste_pot *previous = NULL;
  struct liste_pot *current = premier;

  struct liste_pot *new;

  new=malloc(sizeof(struct liste_pot));

  strncpy(new->login,login,9);
  //  new->next=premier;

  /* Recherche de l'emplacement du nouveau login */
  /* On avance jusqu'a trouve soit la fin, soit un login qui est
     superieur au nouveau */

  while(current != NULL)
    {
	 if(strcmp(login, current->login) > 0)
	   {
		previous = current;
		current = current->next;
	   }
	 else
	   break;
    }

  if(previous == NULL) /* on ajoute le login en tete */
    {
      new->next = current;
      premier = new;
    }
  else
    {
      previous->next = new;
      new->next = current;
    }

  return premier;
}

/* fonction retirant un login dans liste_pot */
struct liste_pot *del_liste_pot(char *login, struct liste_pot *premier)
{
  struct liste_pot *cherche;
  struct liste_pot *erase;
  

  cherche = premier;
  
  if ( ! strcmp(cherche->login,login) )
    {
      premier = cherche->next;
      free(cherche);
    }
  else
    {
      while ( (cherche->next != NULL) && strcmp(cherche->next->login,login) )
	cherche=cherche->next;
      
      if (cherche->next != NULL)
	{
	  erase = cherche->next;
	  cherche->next = erase->next;
	  free(erase);
	}
    }
  
  

  return premier;
}

/* fonction pour tout efface */
void suppr_all_liste_pot(struct liste_pot *premier)
{
  struct liste_pot *erase;
  
  while (premier != NULL)
    {
      erase = premier;
      premier = premier->next;
      free(erase);
    }
  
  return ;
}

/* fonction pour creer un fichier a partir de la liste chainee */
void fichier_liste_pot(char *chemin,struct liste_pot *premier)
{
  FILE *pot;
  
  pot = fopen(chemin, "w");
  erreur(pot);

  while (premier != NULL)
    {
      fprintf(pot , "%s\n" , premier->login);
      premier = premier->next;
    }
  
  fclose(pot);

  return ;
}


/* retourne une chaine de caractre pour l'idle (pas d'allocation */
const char * print_idle(time_t idle_time)
{
  static char idle_char[10];

  /* pour Samba, xdm et erreur de lecture sur la tty */
  if (idle_time < 0)
    return (const char *) "???";
  
  /* juste secondes */
  if (idle_time < 60)
    {
      sprintf (idle_char, "%ds", (int) idle_time);
      return (const char *) idle_char;
    }
  
  /* juste minutes */
  if (idle_time < 60*60)
    {
      sprintf (idle_char, "%dm", (int) (idle_time/60));
      return (const char *) idle_char;
    }

  /* heure + minutes */
  if (idle_time < 60*60*24)
    {
      sprintf (idle_char, "%dh%02d",
	       (int) (idle_time / (60 * 60)),
	       (int) ((idle_time % (60 * 60)) / 60));
      return (const char *) idle_char;
    }
  
  /* jours seulement */
  if (idle_time < 60*60*24*10000)
    {
      sprintf (idle_char, "%dj",
	       (int) (idle_time / (60 * 60 * 24)));
      return (const char *) idle_char;
    }
  
  /* au cas ou il y aurait un grand concours d'idle ;-) */
  /* pas de "s"  ans pour des raisons de taille */
  return (const char *) ">27an"; 
}
